package POE::Component::Leve::patternList;
##############################################################################
=pod

=head1 NAME

patternList  

=head1 DESCRIPTION


Delegator class that POE'izes te patternList implementation

=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 patternList.pm is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=cut

=head3 global stuff

Some global stuff is defined here

=cut

######################################################################
use strict;
use Data::Dumper;

use POE qw/Session/;

#my @wishlist = ();		#list of regexps that the user wants to fetch 
#my %series = ();		#list of regexps and serial numbers that the user wants to fetch 
use vars qw/$VERSION/;

$VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
##############################################################################
=pod

=head1 Methods of this class

=over

=item init

set up the main datastructures of this object, make it ready for the run

=cut

######################################################################
sub init
{
  my($this,$settings,$reporter) = @_;
  $this->{"impl"}->init($settings,$reporter);

  POE::Session->create( object_states => [

      $this =>
      {  _start => "_start",
      _stop  => "_stop",
      save => "save",
      },

  $reporter =>
  {
    _default => "_default",
    "print" => "print",
    "log" => "print_log",
    "debug" => "print_debug",
  }
  ]
    );
}#sub init
##############################################################################
=pod

=item start of the poe session


=cut

######################################################################
sub _start 
{
  my ($this, $kernel, $heap, $dsn, $queries) = @_[OBJECT, KERNEL, HEAP, ARG0, ARG1];
  $this->{"kernel"} = $kernel;
  $kernel->yield('debug',"started series manager");
  #print("_start::PL:deb started series manager\n");
}#sub _start
##############################################################################
=pod

=item stop of the poe session


=cut

######################################################################
sub _stop 
{
  my ($this, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
  $kernel->yield('debug',"stopped series manager");
}#sub _stop 
######################################################################
=pod

=item new 

constructor for the new xchat handler object

=cut

######################################################################
sub new 
{
  my $fs = shift;
  my $serverName = shift;
  my $o_xchatHandler = { };

  bless $o_xchatHandler, "POE::Component::Leve::patternList";
  use POE::Helper::Leve::patternList;
  $o_xchatHandler->{"impl"} =  new POE::Helper::Leve::patternList();
  return $o_xchatHandler;
}# sub new 
##############################################################################
=pod

=item match2pattern

compare the incoming filename to the pattern list, and return the 
pattern on success, the modus tells us if we are in dirscan modus or file
transfer success modus, in the latter case we remove or update the pattern
info

=cut

######################################################################
sub match2pattern
{
  my ($this, $args) = @_;
  return $this->{"impl"}->match2pattern($args);
}#sub match2pattern
######################################################################
=pod

=item load

loading all the relevant stuff

=cut

######################################################################
sub load
{
  my($this) = @_;
  return $this->{"impl"}->load();
}#sub load
######################################################################
=pod

=item loadWishes

loading the wishes database

=cut

######################################################################
sub loadWishes
{
  my($this,$filename) = @_;
  return $this->{"impl"}->loadWishes($filename);
}#sub loadWishes
######################################################################
=pod

=item loadSeries

load a file first col is the series pattern, rest are maxto fetch,
  min-fetched, and allready fetched numbers, if max is set to 0 fetching is
  unlimited, beware no spaces in the pattern!

=cut

######################################################################
sub loadSeries
{
  my($this,$filename) = @_;
  return $this->{"impl"}->loadSeries($filename);
}#sub loadSeries
######################################################################
=pod

=item save

Save the list of series and the whishlist!

needs the settings:

settings->{"seriesFile"}; default: $ENV{HOME}/.leve/leve.series
settings->{"whishesFile"}; default: $ENV{HOME}/.leve/leve.wishes

=cut

######################################################################
sub save
{
  #my($this) = @_;
  my ($this, $kernel) = @_[OBJECT, KERNEL];
  $this->{"kernel"} = $kernel if($kernel);
  return $this->{"impl"}->save();
}#sub saveSeries
######################################################################
=pod

=item delwish

delete pattern no

=cut

######################################################################
sub delwish
{
  my($this,$position) = @_;
  return $this->{"impl"}->delwish($position);
}#sub delwish
######################################################################
=pod

=item seriesAdvance

check series boundaries and adjust them if needed

=cut

######################################################################
sub seriesAdvance
{
  my ($this,$pattern,$episode) = @_;
  return $this->{"impl"}->seriesAdvance($pattern,$episode);
}#sub seriesAdvance
######################################################################
=pod

=item delwish

delete pattern no

=cut

######################################################################
sub delwish
{
  my ($this,$position) = @_;
  return $this->{"impl"}->delwish($position);
}#sub delwish
######################################################################
=pod

=item patternrevision

switch patter active chars to perlish and non greedy patterns

=cut

######################################################################
sub patternrevision
{
  my ($this,$line) = @_;
  return $this->{"impl"}->patternrevision($line);
}#sub patternrevision
######################################################################
=pod

=item series

list/add series

=cut

######################################################################
sub series
{
  my  ($this,$line) = @_;
  return $this->{"impl"}->series($line);
}# sub series
######################################################################
=pod

=item alias

list/add alias

=cut

######################################################################
sub alias
{
  my  ($this,$line) = @_;
  return $this->{"impl"}->alias($line);
}# sub alias
######################################################################
=pod

=item request

issue the actual request banner on the ad-channels

=cut

######################################################################
sub request
{
  my  ($this,$line) = @_;
  return $this->{"impl"}->request($line);
}#sub request
######################################################################
=pod

=item delseries

delete the series given as parameter, cut and paste from the series
listing

=cut

######################################################################
sub delseries
{
  my  ($this,$pattern) = @_;
  return $this->{"impl"}->delseries($pattern);
}#sub delseries
######################################################################
=pod

=item addepisode

in case you fetched an episode by other means, adjust the filter of the
script to drop fetching of that particular episode(s)",

=cut

######################################################################
sub addepisode
{
  my  ($this,$line) = @_;
  return $this->{"impl"}->addepisode($line);
}#sub addepisode
######################################################################
=pod

=item delepisode 

in case a fetched episode reveals itself as somehow inappropriated, adjust
the filter of the script to restart fetching of that particular episode(s)",

=cut

######################################################################
sub delepisode 
{ 
  my  ($this,$line) = @_;
  return $this->{"impl"}->delepisode($line);
}#sub delepisode
######################################################################
=pod

=item seriesRescedule

check series boundaries and adjust them if needed

=cut

######################################################################
sub seriesRescedule
{
  my  ($this,$pattern,$episode) = @_;
  return $this->{"impl"}->seriesRescedule($pattern,$episode);
}#sub seriesRescedule
######################################################################
=pod

=item modseries

set the importance you give to a given series, mostly determinant for the
request banner generation",

=cut

######################################################################
sub modseries
{
  my  ($this,$line) = @_;
  return $this->{"impl"}->modseries($line);
}#sub seriesprio
######################################################################
=pod

=item listwishes

list the wishes

=cut

######################################################################
sub listwishes
{
  my  ($this,$pattern) = @_;
  return $this->{"impl"}->listwishes($pattern);
}#sub listwishes
######################################################################
=pod

=item delwish

delete pattern no

=cut

######################################################################
sub delwish
{
  my  ($this,$position ) = @_;
  return $this->{"impl"}->delwish($position);
}#sub delwish
######################################################################
=pod

=item print

local print method that will seeing the availability launch a POE event or print out manually

=cut

######################################################################
sub print
{
  my  ($this,$msg ) = @_;

  my $kernel = $this->{"kernel"};
  if($kernel)
  {
    $kernel->post('print',$msg);
  }# if($kernel)
  else
  {
    print("NOHANDLER!!!: $msg\n");
  }# else
}# sub print
######################################################################
=pod

=item rawnames

generate a list of all patterns from the series and the wishlist

=cut

######################################################################
sub rawnames
{
  my($this) = @_;
  return $this->{"impl"}->rawnames();
}#sub rawnames
######################################################################
=pod

=item getSerie

getter for the series hash, since we have plain entries and aliasies, lets have
one place were we handle this difference

=cut

######################################################################
sub getSerie
{
  my($this,$name) = @_;
  return $this->{"impl"}->getSerie($name);
}#sub getSerie
######################################################################
=pod

=item delSerie

deleting one series means deleting the real record and all associated aliases

=cut

######################################################################
sub delSerie
{
  my($this,$name) = @_;
  return $this->{"impl"}->delSerie($name);
}#sub delSerie
1
__END__


=back

=head1 THANKS

I. Wronsky came up with this infernal concept and kludged the first 1.x
versions together. This gave B. Boettcher the incentive to begin this work, the
famous itching paradigm!

=head1 SEE ALSO

http://www.geocities.com/iwronsky/
http://poe.perl.org/

=cut

