#!/usr/bin/perl -w
use Data::Dumper;
##############################################################################
=pod

=head1 NAME

visualShell

=head1 DESCRIPTION

puts up a terminal interface to the leve system

=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 visualShell.pl is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=cut

##############################################################################


IRC::print "Loading spamignore.pl\n";
POE::Component::Leve::xchatHandler::load_si();
IRC::print "Registering Handlers...\n";
IRC::add_command_handler("shell", "POE::Component::Leve::xchatHandler::si_command");

IRC::add_message_handler("PRIVMSG", "POE::Component::Leve::xchatHandler::privmsg_handler");
IRC::add_print_handler("Private Message to Dialog", "POE::Component::Leve::xchatHandler::privmsg_handler");

print "\n\n---- begin of tests!!! ---\n";

POE::Component::Leve::xchatHandler::si_command("help");
print "---- help with arg  ---\n";
POE::Component::Leve::xchatHandler::si_command("help list");
print "---- list command  ---\n";
POE::Component::Leve::xchatHandler::si_command("list");



package POE::Component::Leve::xchatHandler;

use warnings;
use strict;

use Data::Dumper;

our(@spam_ignore,$cmdList);
@spam_ignore = ();

##########################################################################
=pod

=item spam_handler

return 1 if the incoming line should be ignored, undef otherwise

=cut

##########################################################################
sub spam_handler
{
  ($nick,$msg) = @_;

  foreach my $isspam (@spam_ignore) 
  {
    return 1 if ($msg =~ /$isspam/i) ;
  }# foreach (@webspam_ignore) 
}# sub spam_handler

##########################################################################
=pod

=item privmsg_handler

parse privmessage type of lines

=cut

##########################################################################
sub privmsg_handler
{
  my $line = shift(@_);
  if( $line =~ /:(\S+)!(\S+)@(\S+) (\S+) (\S+) :(.*)/)
  {
    ($nick,$ident,$host,$cmd,$to,$msg) = ($1,$2,$3,$4,$5,$6);
    my $ownnick  = IRC::get_info(1);
    return spam_handler($nick, $msg) if($ownnick =~ /\Q$to\E/);
  }#if( $line =~ /:(\S+)!(\S+)@(\S+) (\S+) (\S+) :(.*)/)
  return 0;
}# sub privmsg_handler

##########################################################################
=pod

=item load_si 

start/initialize this script

=cut

##########################################################################
sub load_si 
{
  $cmdList = 
  {
    "load" 	=>	{
      "cmd"	=>	"load_si",
      "help"	=>	"load ignore list"},
      "clear"	=>	{
	"cmd"	=>	"clear",
	"help"	=>	"clear ignore list"},
	"save"	=>	{
	  "cmd"	=>	"save_si",
	  "help"	=>	"save ignore list"},
	  "add"	=>	{
	    "cmd"	=>	"add",
	    "help"	=>	"/si add <pattern>, add pattern to ignore list"},
	    "replace"	=>	{
	      "cmd"	=>	"replace",
	      "help"	=>	"/si replace <pattern-num> <pattern>, replace pattern number pattern-num with a new pattern"},
	      "list"	=>	{
		"cmd"	=>	"list",
		"help"	=>	"list actual patterns"},
		"del"	=>	{
		  "cmd"	=>	"del",
		  "help"	=>	"delete pattern number pattern-num"},
		  "help"	=>	{"cmd"	=>	"help",
		    "help"	=>	"issue this help, or with a command name the help of a given command"},
  };
  open(FIN, (glob("~/.spam-ignore.txt"))[0])
    or return;
  @spam_ignore = ();
  while (<FIN>) 
  {
    chomp;
    push @spam_ignore, $_;
  }# while (<FIN>) 
  close FIN;
  IRC::print "--spam ignore list loaded--\n";
}# sub load_si 

##########################################################################
=pod

=item save_si 

save the spam regexps

=cut

##########################################################################
sub save_si 
{
  open(FOUT, ">", (glob("~/.spam-ignore.txt"))[0])
    or return;
  foreach (@spam_ignore) {
    print FOUT "$_\n";
  }
  close FOUT;
  IRC::print "--spam ignore list saved--\n";
}# sub save_si 

#/si load
#/si clear
#/si save
#/si add <pattern>
#/si replace <pattern-num> <pattern>
#/si list
#/si del <pattern-num>
##########################################################################
=pod

=item si_command 

shell

=cut

##########################################################################
sub si_command 
{
  my $line = shift(@_);
  $line =~ /^\s*(\S+)\s*(.*)$/;
  my($cmd,$rest) = (lc($1),$2);
  #print "cmdList = ".Dumper($cmdList)."\n";

  if( $cmdList->{$cmd})
  {
    my $func =  "POE::Component::Leve::xchatHandler::".$cmdList->{$cmd}{"cmd"};
    #print "calling $func($rest)\n";
    return &$func($rest);
  }# if( $cmdList->{$cmd})
  else
  {
    return help();
  }#else
}# sub si_command 
##########################################################################
=pod

=item clear 

clear spam ignore list

=cut

##########################################################################
sub clear 
{
  @spam_ignore = ();
  IRC::print "--spam ignore list cleared--\n";
}# sub clear 
##########################################################################
=pod

=item add

add a pattern to the list

=cut

##########################################################################
sub add
{
  my $remain = shift;
  push @spam_ignore, $remain;
  IRC::print "--spam ignore added $remain--\n";
}# sub add
##########################################################################
=pod

=item replace

replace a pattern from the existing list

=cut

##########################################################################
sub replace
{
  my $remain = shift;
  $remain =~ /(\d+)\s+(.*)$/i;
  ($index, $remain) = ($1, $2);
  if ($index && ($index >= 0 && $index <= $#spam_ignore)) 
  {
    $spam_ignore[$index] = $remain;
  }
  IRC::print "--spam ignore replaced pos $index with $remain--\n";
}# sub replace
##########################################################################
=pod

=item list

list the actual list of spam ignore patterns

=cut

##########################################################################
sub list
{
  my $count;
  foreach (@spam_ignore) 
  {
    IRC::print $count+0 . " - $_\n";
    $count++;
  }
  IRC::print "--End of Spam Ignore List--\n";
}# sub list
##########################################################################
=pod

=item del

delete a pattern from the list

=cut

##########################################################################
sub del
{
  my $index = shift;
  $index =~ /(\d+)/i;
  $index = $1;
  if (length($index) && ($index >= 0 && $index <= $#spam_ignore)) 
  {
    splice @spam_ignore, $index+0, 1;
  }# if (length($index) && ($index >= 0 && $index <= $#spam_ignore)) 
  IRC::print "--spam ignore deleted pos $index--\n";
}# sub del
##########################################################################
=pod

=item help

issue the help for this script

=cut

##########################################################################
sub help
{
  my $topic = shift;
  IRC::print "--spam ignore help--\n";
  if($topic && $cmdList->{$topic})
  {
    IRC::print "--$topic:".$cmdList->{$topic}{"help"}."\n";
  }#if($topic)
  else
  {
    IRC::print "--for specific help supply a valid si command as argument--\n";
    IRC::print "--valid arguments/commands are:--\n";
    my $msg = "";
    foreach my $cmd (sort(keys(%$cmdList)))
    {
      $msg .= "$cmd ";	
    }# foreach my $cmd (sort(keys($cmdList)))
    IRC::print "$msg\n";
  }#else
}#sub help
1
__END__


=back

=head1 THANKS

I. Wronsky came up with the concept for leverloda
This gave B. Boettcher the incentive to begin this work, the
famous itching paradigm!

=head1 SEE ALSO

http://www.geocities.com/iwronsky/
http://poe.perl.org/

=cut

