#!/usr/bin/perl
##############################################################################
=pod

=head1 NAME

configLoader 

=head1 DESCRIPTION

parses command line and supplyed files for arguments, options and settings

=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 patternQuery.pl is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=cut

##############################################################################
package POE::Helper::configLoader;
use warnings;
use strict;

use Data::Dumper;

use constant FALSE => 0;
use constant TRUE => 1;
######################################################################
=pod

=item new 

constructor for the load configuration object

=cut

######################################################################
sub new 
{
  my ($fs,$hashref) = @_;
  my $obj_ref = { };
  $obj_ref->{"settings"} = $hashref if ($hashref);

  bless $obj_ref, "POE::Helper::configLoader";
  return $obj_ref;
}# sub new 
######################################################################
=pod

=item setValidKeys 

getter/setter for the hash of keys that should be accepted through shell
arguments, if this isn't present any argument will be accepted.

=cut

######################################################################
sub setValidKeys 
{
  my ($this,$hashref) = @_;
  if($hashref && ref($hashref) eq 'HASH')
  {
    $this->{"allowedkeys"} = $hashref;
  }# if($hashref)
  else
  {
    print("returning available allowed keys:".Dumper($this->{"allowedkeys"})."\n");
    return $this->{"allowedkeys"};
  }# else
}# sub setValidKeys 
######################################################################
=pod

=item container 

getter/setter for the hash of settings we are manipulating.

=cut

######################################################################
sub container 
{
  my ($this,$hashref) = @_;
  if($hashref && ref($hashref) eq 'HASH')
  {
    $this->{"settings"} = $hashref;
  }# if($hashref)
  else
  {
    return $this->{"settings"};
  }# else
}# sub container 
######################################################################
=pod

=item load 

load a file with a hash reference in perl format

=cut

######################################################################
sub load 
{
  my ($this,$filename) = @_;
  my $tmphash = do($filename);
  #print("load fetched: ".Dumper($tmphash)."\n");
  foreach my $entry (keys(%$tmphash))
  {
    $this->{"settings"}->{$entry} = $tmphash->{$entry};
  }# foreach my $entry (keys(%$tmphash))
  #print("finished load set ".Dumper($this->{"settings"})."\n");
  return $this->{"settings"};
}# sub load 
######################################################################
=pod

=item parseArgs 

parse the incoming arguments, if allowedkeys hash is present, it is used to
translate the keys to the sttings used in the program, otherwise the keyname is
used as parameter name. Also if the allowedkeys hash is present only the keys
psecifiyed in there are allowed. 

using the hash implies: the keys are still with the prepended '-''s, if after
the '-'s a no is detected it is thought as a negation setting if there isn't a
valid value following.

=cut

######################################################################
sub parseArgs
{
  my($this,$arguments) = @_;

  while(scalar(@$arguments) >0)
  {
    my $key = shift(@$arguments);
    $key = lc($key);
    my $negate = FALSE;
    if($key =~ /(-+)no(.*)/)
    {
      $key = $1.$2;
      $negate = TRUE;
    }# if($key =~ /(-+)no(.*)/)
    
    if($this->{"allowedkeys"})
    {
      if($this->{"allowedkeys"}->{$key} || $this->{"allowedkeys"}->{"no".$key})  
      {
	my $value = $this->fetchNext($arguments,$negate);
	if($negate && $value ne TRUE)
	{
	  $key = "no".$key;
	}# if($negate && $value ne TRUE)
	$this->{"settings"}->{$this->{"allowedkeys"}->{$key}} = $value;
      }# if($this->{"allowedkeys"}->{$key})  
      else
      {
	print("invalid argument submitted, allowed are ".Dumper($this->{"allowedkeys"})."\n");
      }# else
    }# if($this->{"allowedkeys"})
    else
    {
      $key =~ s/^-+//;
	my $value = $this->fetchNext($arguments,$negate);
	if($negate && $value ne TRUE)
	{
	  $key = "no".$key;
	}# if($negate && $value ne TRUE)
	$this->{"settings"}->{$key} = $value;
    }# else
  }# while(scalar(@$arguments) >0)
}#sub parseArgs
######################################################################
=pod

=item fetchNext 

check out the next argument, if it isn't prepended with - it should be a value
that we return, otherwise we push back the thing and return the logical
attribute.

=cut

######################################################################
sub fetchNext
{
  my($this,$arguments,$bool) = @_;

    my $value = shift(@$arguments);
    if($value =~ /^-/)
    {
      unshift(@$arguments,$value);
      return $bool;
    }# if($key =~ /(-+)no(.*)/)

    return $value;
}#sub fetchNext
1
__END__


=back

=head1 THANKS

=head1 SEE ALSO

http://poe.perl.org/

=cut

