#!/usr/bin/perl

use warnings;
use strict;

sub BEGIN
{
  push (@INC,$ENV{HOME}.'/download/leverloda/goanseech')
    if(!( (join(' ',@INC)) =~ /goanseech/));
}

# Gtk2 support is enabled if the Gtk2 module is used before POE itself.

use Gtk2;
#use Gnome;

use POE;

use Data::Dumper;
use POE::Filter::Reference;
use POE::Component::Client::TCP;
use POE::Component::goanseech::seriesConnector;

use POE::Helper::configLoader;

use Gtk2::SimpleMenu;
use Gtk2::LogBuffer;
use constant FALSE => 0;
use constant TRUE => 1;

##############################################################################
=pod

=head1 NAME

gtkGUI

=head1 DESCRIPTION

connects to a patternServer and allows to manipulate the data of
that server

=head1 AUTHOR

Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

Copyright (c) 2002 Bruno Boettcher

gtkGUI.pl is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.

This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=head3 instantiating a TCP client

  Create a client that sends and receives Perl data structures.
  Upon connecting with the server, it sends a list of things to
  process.  The server responds with a hash of a few things.  Upon
  receipt of that hash, the client displays its contents and exits.

=cut

##############################################################################
my $VERSION = '$Revision: 1.8 $';
$VERSION =~ s/.*(\d+\.\d+).*/$1/;

my $allowedSettings = { "--config" => "rootpath" };
my $settings = {
  "rootpath" => $ENV{HOME}."/.goanseech/",
  "bufferlength" => 500,
};

my $configurator = new POE::Helper::configLoader($settings);
$configurator->setValidKeys($allowedSettings);

$settings->{"rootpath"} .= "/" if(!($settings->{"rootpath"} =~ /\/$/));

$configurator->load($settings->{"rootpath"}."settings.pm");

my %series = ();
my %aliases = ();

my $table = [];
my $buildup = 1;
#my @linesarray;
my $transfers = {};
my $windows = {};

my $printbuffer = Gtk2::LogBuffer->new($settings);
$windows->{"progresssize"} = Gtk2::SizeGroup->new ('horizontal');

#make the connection to the patternServer and retrieve the series list
print( "firing up the connection\n" );
my $querySession = "seriesClient";
my $seriesQuery = new POE::Component::goanseech::seriesConnector($querySession);
my $callbacks = 
{
  "defaultSession" => "gtkSeries",
  "defaultEvent" => "default",
  "msgevent" => "default",
};
$seriesQuery->init($settings, $callbacks);

# Create the session that will drive the user interface.

POE::Session->create
( inline_states =>
  { _start	=> \&ui_start,
  ev_count	=> \&ui_count,
  ev_clear	=> \&ui_clear,
  ev_change	=> \&ui_change,
  ev_delete	=> \&ui_delete,
  ev_addeps	=> \&addEpisodes,
  fillup	=> \&create_window,
  default	=> \&default,
  "getSeries"	=> \&gotMessage,
  "refresh"	=> \&refresh,
  "updatetransfers"	=> \&updatetransfers,
  }
);

# Run the program until it is exited.

print("starting the kernel\n");
$poe_kernel->run();
exit 0;

##############################################################################
=pod

=item _start

to keep this session alive set an alias

=cut
######################################################################
sub ui_start
{
  my ( $kernel, $arg ) = @_[ KERNEL, ARG0 ];
  $_[KERNEL]->alias_set("gtkSeries");
  #$kernel->yield("fetchURLs");  
  print("starting sessions and setting aliases\n");
  my @request = ( "alias", "series"); 
  #reset the listing of pages to seach, since we are asking for a whole new bunch of pages...
  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "getSeries",
  };
  $_[KERNEL]->post($querySession, "doneParsing", $job);
}# sub ui_start
##############################################################################
=pod

=item create_window 

create the user interface when the session starts.  This assumes
some familiarity with Gtk2.  ui_start() illustrates four important
points.

- Gtk2 events do not require require a main window.  It is therefore the
responsibility of the programmer to create a main window (or not) herself.
POE::Kernel's signal_ui_destroy() method attaches a IDESTROY signal to the
destruction of a window.  In this case, closing the main window signals the
program to shut down.

- Widgets we need to work with later, such as the counter display, must be
stored somewhere.  The heap is a convenient place for them.

- Gtk2 widgets expect callbacks in the form of coderefs.  The session's
postback() method provides coderefs that post events when called.  The Button
created in ui_start() fires an "ev_clear" event when it is pressed.

- POE::Kernel methods such as yield(), post(), delay(), signal(), and select()
  (among others) work the same as they would without Gtk2.  this feature makes
  it possible to write back end sessions that support multiple GUIs with a
  single code base.

=cut

##############################################################################
sub create_window 
{
  my ( $kernel, $session, $heap ) = @_[ KERNEL, SESSION, HEAP ];

  print("creating window\n");
  $heap->{main_window} = Gtk2::Window->new("toplevel");
  $heap->{main_window}->set_title("Leech system! Version");;
  $kernel->signal_ui_destroy( $heap->{main_window} );
  $heap->{main_window}->set_default_size(550, 500 );

  if($buildup)
  {
    $buildup = 0;
    my $fixed = new Gtk2::VBox(0,0);
    $heap->{main_window}->add( $fixed);
    $fixed->show;

    my @menu_tree = (
        '_File' => {
        item_type => '<Branch>',
        children => [
        'Load' => {
        callback => sub{ ui_load($kernel);},
        #callback => \&ui_load,
        #user_data => $kernel,
        #callback_action => 1,
        },
    'Save' => {
      callback => sub{ ui_save($kernel);},
      #callback => \&ui_save,
      #user_data => $kernel,
      #callback_action => 2,
    },
    'Reload' => {
      callback => sub{ ui_reload($kernel);},
      # callback => \&ui_reload,
      #user_data => $kernel,
      #callback_action => 3,
    },
    'Server Shutdown' => {
      callback => sub{ 
        my @request = ( "shutdown");
        my $job = {
          "requests"	=> \@request,
          "session"		=> "gtkSeries",
          "event"		=> "default",
          "msgevent"	=> "default",
        };
        $kernel->post($querySession, "doneParsing", $job);
        ui_quit($kernel);
      },
    },
    'Quit' => {
      callback => sub{ ui_quit($kernel);},
      #user_data => $kernel,
      #callback_action => 4,
    },
    ],
        },
    '_Edit' => {
      item_type => '<Branch>',
      children => [
        'settings' => {
          callback => sub{ ui_settings($kernel, $heap);},
        },
      'logging on' => {
        callback => sub{ 
          my @request = ( "printhandler on");
          my $job = {
            "requests"	=> \@request,
            "session"		=> "gtkSeries",
            "event"		=> "default",
            "msgevent"	=> "default",
          };
          $kernel->post($querySession, "doneParsing", $job);
        },
      },
      'logging off' => {
        callback => sub{ 
          my @request = ( "printhandler off");
          my $job = {
            "requests"	=> \@request,
            "session"		=> "gtkSeries",
            "event"		=> "default",
            "msgevent"	=> "default",
          };
          $kernel->post($querySession, "doneParsing", $job);
        },
      },
      'logging window' => {
        callback => sub{ ui_statusdialog($kernel); },
      },
      ],
    },
    '_Help' => {
      item_type => '<LastBranch>',
      children => [
        'About' => {
          callback => sub{ ui_about($kernel, $heap);},
          #user_data => [ $kernel, $heap ],
          #callback_action => 5,
        },
      ],
    },
    );

    my $menu = Gtk2::SimpleMenu->new (
        menu_tree => \@menu_tree,
        #default_callback => \&default_callback,
        #user_data => 'user_data',
        );

    $fixed->pack_start($menu->{widget},0,0,0);




    $heap->{scroll_window} = Gtk2::ScrolledWindow->new();
    #$heap->{scroll_window}->set_usize(430, 500 );
    $fixed->add($heap->{scroll_window});
    #$fixed->pack_start($heap->{scroll_window},1,1,0);
    #$fixed->add($heap->{scroll_window});
    #$heap->{scroll_window}->set_uposition(0, 24 );

    my $inputFields = new Gtk2::HBox(0,0);
    $fixed->pack_end($inputFields,0,0,0);
    my $patternBox = new Gtk2::Entry();
    my $adjMin = new Gtk2::Adjustment(1, 0, 1025, 1, 10, 10 );
    my $adjMax = new Gtk2::Adjustment(1024, 0, 1025, 1, 10, 10 );
    my $spinMin = new Gtk2::SpinButton($adjMin,1,0);
    my $spinMax = new Gtk2::SpinButton($adjMax,1024,0);
    my $otherBox = new Gtk2::Entry();

    $heap->{"newinput"} = {
      "pattern" => $patternBox,
      "min"	=> $adjMin,
      "max"	=> $adjMax,
      "other"	=> $otherBox
    };

  $otherBox->signal_connect( 'activate' => 
      sub { newSeries($kernel,$heap);});
    #$adj->signal_connect( 'value_changed' => 
        #   sub { min_change(@_, $line->{"name"}, "min",$kernel);});
    $inputFields->add($patternBox);
    $inputFields->add(new Gtk2::Label());
    $inputFields->add($spinMin);
    $inputFields->add($spinMax);
    $inputFields->add($otherBox);


    #$fixed->add($heap->{scroll_window});

    my $box = Gtk2::Table->new( 4, scalar(keys(%series)) +1 );
    #$box->set_usize(486, 100 );
    #$viewport->add($box);
    #$heap->{scroll_window}->add($viewport);
    #$heap->{scroll_window}->add_with_viewport($box);
    #$fixed->add($box);
    #$viewport->add($box);
    $heap->{scroll_window}->add_with_viewport($box);
    #$box->set_uposition(0, 24 );
    #$box->set_usize(80, 96 );
    #$heap->{scroll_window}->add(Gtk2::Viewport::new($box,0,0));
    #$box->set_uposition(0, 24 );
    $box->show();

    #print("ui_start! series = ".Dumper(\@series)."\n");

    my $index = 0;
    my $next = $index+1;
    my @titles = qw(Pattern delete min max other-episodes);
    my @aligns = (1,0,0,0,1);
    my $x = 0;
    foreach my $title (@titles)
    {
      #my $markup = "<span weight=\"bold\" size=\"large\">$title</span>";
      my $markup = "<span weight=\"bold\">$title</span>";

      #my $label = Gtk2::Label->new($title);
      my $label = Gtk2::Label->new();
      if($aligns[$index])
      {
        $label->set_alignment(0,0);
      }# if($aligns[$index])
      $label->set_markup($markup);
      $label->set_justify('center');
      $box->attach($label, $x, ($x + 1), $index, $next,['expand','fill'] ,['expand','fill'] , 0, 0);


      $label->show();
      $x++;
    }# foreach my $title (@titles)

    $index++;

    foreach my $pattern (sort(keys(%series)))
    {
      my $line = $series{$pattern};
      push(@$table, add_table_line($box,$line,$index,$kernel));
      $index++;

    }# foreach my $line (@series)

    #$heap->{counter}       = 0;
    #$heap->{counter_label} = Gtk2::Label->new( $heap->{counter} );
    ##$box->pack_start( $heap->{counter_label}, 1, 1, 0 );
    #Gtk2::Table::attach( $box,$heap->{counter_label}, 1, 2, 0, 1, 0, 0, 0, 0);
    #$heap->{counter_label}->show();

    #my $button = Gtk2::Button->new("Clear");
    #$button->signal_connect( "clicked", $session->postback("ev_clear") );
    ##$box->pack_start( $button, 1, 1, 0 );
    #Gtk2::Table::attach( $box,$button, 0, 2, 1, 2, 0, 0, 0, 0);
    #$button->show();

    #$heap->{scroll_window}->show();
    #$heap->{main_window}->show();
    $heap->{main_window}->show_all;
    print( "windows should show up\n" );

    #$kernel->yield("ev_count");
  }# if(scalar($table) <= 0)
  else
  {
    print ("reloading...\n");

    my @patterns = sort(keys(%series));
    #print("dataset: ".Dumper(\@patterns)."\n");
    print("tables: ".scalar(@patterns)." vs ".scalar(@$table)."\n");
    foreach my $table_line (@$table)
    {
      my $pattern = shift(@patterns);
      my $line = $series{$pattern};
      print("refreshing line $pattern with ".Dumper($line)."\n");
      for my $td (qw(name min max other))
      {
        if($line->{$td})
        {
          $table_line->{$td}->set_text($line->{$td});
        }# if(!$line)
        #TODO disconnect signal handler and reconnect it....
      }# for my $td (qw(name min max other))

    }# foreach my $line (@series)
    print("finished replenishing: ".scalar(@$table)." vs ".scalar(@patterns)."\n");
    #foreach my $pattern (@patterns)
    #{
      #  push(@$table, add_table_line($box,$line,$index,$kernel));
      #$index++;
      #}# foreach my $line (@series)
  }# else
}

##############################################################################
=pod

=item ui_count 

Handle the "ev_count" event by increasing a counter and displaying
its new value.

=cut

##############################################################################
sub ui_count 
{
  my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
  $heap->{counter_label}->set_text( ++$_[HEAP]->{counter} );
  $kernel->yield("ev_count");
}
##############################################################################
=pod

=item ui_clear 

Handle the "ev_clear" event by clearing and redisplaying the
counter.

=cut

##############################################################################
sub ui_clear 
{
  $_[HEAP]->{counter} = 0;
}
##############################################################################
=pod

=item ui_change 

some changes were made to the boundaries of some series reflect those changes
to the server

=cut

##############################################################################
sub ui_change 
{
  my($type,$pattern,$value) = @_[ARG0,ARG1,ARG2];
  #clear$_[HEAP]->{counter} = 0;
  my $kernel = $_[KERNEL];

  #print("ui_change: 	pat:".$_[ARG1]." field:".$_[ARG0]." value:".$_[ARG2]."\n");
  my @request;
  if($type eq "min")
  {
    if($series{$pattern}->{$type} > $value)
    {
      #print("delepisode of $pattern ...\n");
      $series{$pattern}->{$type} -- ;
      @request = ( 
          "delepisode $pattern ".$series{$pattern}->{$type},
          );
    }# if($series{$pattern}->{$type} > $value)
    else
    {
      #print("addepisode for $pattern ... \n");
      @request = ( 
          "addepisode $pattern ".$series{$pattern}->{$type},
          );
      $series{$pattern}->{$type} ++ ;
    }# else
  }# if($type eq "min")
  elsif($type eq "max" || $type eq "prio" || $type eq "tag")
  {
    #print("modseries for $pattern ... \n");
    @request = ( 
        "modseries $pattern $type $value",
        );
    $series{$pattern}->{$type} = $value ;
  }# else
  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put 'delete $pattern'\n");
  }# else
}# sub ui_change 
##############################################################################
=pod

=item addEpisodes 

some intermediary episodes were added

=cut

##############################################################################
sub addEpisodes 
{
  my($pattern,$value) = @_[ARG0,ARG1];
  #clear$_[HEAP]->{counter} = 0;
  my $kernel = $_[KERNEL];

  my $oldvalues = $series{$pattern}{"other"};
  #print("addEpisodes 	pat:$pattern value:$value vs $oldvalues\n");
  my @oldarray = split(/ /,$oldvalues);
  my @newarray = split(/ /,$value);
  my (@addarray, @rmarray);

  my %newhash = ();
  my %oldhash = ();
  foreach my $subval (@oldarray)
  {
    #print("inserting presence of $subval\n");
    $oldhash{$subval} = 1; 
  }# foreach my $subval (@oldarray)

  foreach my $subval (@newarray)
  {
    if($oldhash{$subval} && $oldhash{$subval} == 1) 
    {
      $oldhash{$subval} = 0; 
    }# if($oldhash{$subval} == 1) 
    else
    {
      #we have a value that is not in the oldhash
      push(@addarray,$subval);
    }# else
  }# foreach my $subval ($value)

  foreach my $subval (keys(%oldhash))
  {
    push(@rmarray,$subval) if($oldhash{$subval} == 1);
  }# foreach my $subval (keys(%oldhash))
  
  my @request;
  
      push(@request, "addepisode $pattern ".join(' ',@addarray)) if(scalar(@addarray) > 0);
      push(@request, "delepisode $pattern ".join(' ',@rmarray)) if(scalar(@rmarray) > 0);

      #print("prepared request:".Dumper(\@request)."\n");

  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put 'addEpisodes $pattern'\n");
  }# else

  $series{$pattern}{"other"} = $value;
}# sub addEpisodes 
##############################################################################
=pod

=item min_change 

callback for the GTK stuff, this method is called when an event occurs on one of the spin buttons

=cut

##############################################################################
sub min_change 
{
  my($adj,$pattern,$type,$kernel) = @_;
  #print("min_change ".Dumper(\@_)."\n");
  my $newval = $adj->get_value();
  $kernel->post("gtkSeries", "ev_change", $type, $pattern, $newval) ;
}
##############################################################################
=pod

=item add_episodes

callback for the GTK stuff, this method is called when an event occurs on the additional aps field

=cut

##############################################################################
sub add_episodes 
{
  my($adj,$pattern,$kernel) = @_;
  #print("min_change ".Dumper(\@_)."\n");
  my $newval = $adj->get_text();
  $kernel->post("gtkSeries", "ev_addeps", $pattern, $newval) ;
}
##############################################################################
=pod

=item delseries 

callback for the GTK stuff, this method is called when an event occurs on one of the spin buttons

=cut

##############################################################################
sub delseries 
{
  my($adj,$pattern,$kernel) = @_;
  #print("min_change ".Dumper(\@_)."\n");
  $kernel->post("gtkSeries", "ev_delete", $pattern) ;
}
##############################################################################
=pod

=item ui_quit 

close down the GUI

=cut

##############################################################################
sub ui_quit 
{
  my($kernel) = @_;
  #foreach my $arg (@_)
  #{
    #  print("ui_quit arg: $arg of type ".ref($arg)."\n");
    #}# foreach my $arg (@_)
  print("normal shutdown\n");

  if($kernel)
  {
    #stop the print forwarding
    my @request = ( "printhandler");
    my $job = {
      "requests"	=> \@request,
      "session"		=> "gtkSeries",
      "event"		=> "default",
      "msgevent"	=> "default",
    };
    $kernel->post($querySession, "doneParsing", $job);
    #shutdown the link
    $kernel->post($querySession, "shutdown");
    #shutdown the local Gtk eventloop
    $kernel->post("gtkGUI", "shutdown");
  }
  else
  {
    print("no kernel defined can't call shutdown on socket\n");
  }# else
  print("ui_quit exiting\n");
  exit(0);
}
##############################################################################
=pod

=item ui_load 

tell the server to reload its files

=cut

##############################################################################
sub ui_load 
{
  my($kernel) = @_;
  #print("reloading\n");

  my @request = ( 
      "load",
      );

  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put 'load'\n");
  }# else
}
##############################################################################
=pod

=item ui_save 

tell the server to save its files

=cut

##############################################################################
sub ui_save 
{
  my($kernel) = @_;
  #print("saving\n");

  my @request = ( 
      "save",
      );
  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put 'save'\n");
  }# else
}
##############################################################################
=pod

=item ui_reload 

tell the server to resend the listing

=cut

##############################################################################
sub ui_reload 
{
  my( $kernel) = @_;
  #print("reloading\n");

  my @request = ( 
      "series",
      );

  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put 'reload'\n");
  }# else
}
##############################################################################
=pod

=item ui_settings 

tell the server to send over the actual settings

=cut

##############################################################################
sub ui_settings 
{
  my( $kernel) = @_;
  #print("reloading\n");

  my @request = ( 
      "set",
      );

  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put 'reload'\n");
  }# else
}# sub ui_settings 
##############################################################################
=pod

=item ui_delete 

tell the server to delete a series

=cut

##############################################################################
sub ui_delete 
{
  my($kernel,$pattern) = @_[KERNEL, ARG0];
  #my($adj,$pattern,$type,$kernel) = @_;
  #print("min_change ".Dumper(\@_)."\n");

  my @request = ( 
      "delseries $pattern",
      );
  my $job = {
    "requests"	=> \@request,
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put 'delete $pattern'\n");
  }# else
}
##############################################################################
=pod

=item ui_about 

generate an about message

=cut

##############################################################################
sub ui_about 
{
  my($kernel,$heap) = @_;
  #print("about\n");
  #
  my $pixbuf = undef;
  my $filename = "logo.png";

  #my $dialog = Gtk2::MessageDialog->new($heap->{main_window},
      #    [qw/modal destroy-with-parent/],
      #    'info',
      #    'ok',
      #    "Goanseech Gtk2 Frontend\nVersion $VERSION\nAuthor bboett at adlp org");

  #$dialog->set_icon($pixbuf);
  #$dialog->run;
  #$dialog->destroy;
  if(open(FILE,"CHANGELOG"))
  {
    $VERSION = <FILE>;
    $VERSION =~ s/.*(\d+\.\d+).*/$1/;
    close(FILE);
  }# if(fopen(FILE,"CHANGELOG"))

  my $about = new Gtk2::Dialog;
  my $label = new Gtk2::Label;
  my $markup = <<"    END";

  <span weight="bold" size="large">Goanseech</span>
    v$VERSION

    Make your anime-(or other)-series-leech experience
    easier and automatic

    Copyright (c) 2003 Bruno Bttcher
    bboett at adlp org

    END

    $markup =~ s/\n */\n/gs;
  $label->set_markup($markup);
  $label->set_justify('center');

  my $image = Gtk2::Image->new_from_file ($filename);


  $about->set_title('');
  $about->set_icon ($image->get_pixbuf);
  $about->set_border_width(8);
  $about->set_modal(TRUE);
  $about->set_resizable(FALSE);
  $about->add_buttons('gtk-ok' => 0);
  $about->vbox->set_spacing(8);
  $about->vbox->pack_start(
      $image,
      FALSE, FALSE, 0);
  $about->vbox->pack_start($label, TRUE, TRUE, 0);
  $about->show_all;
  $about->signal_connect(response => sub { $about->destroy(); });
}# sub ui_about 
##############################################################################
=pod

=item add_table_line 

add a line to the actual table

=cut

##############################################################################
sub add_table_line
{
  my ($box, $line, $index,$kernel) = @_;
  my $table_line = {};
  my $next = $index + 1;

  my $x = 0;
  my $labelText = $line->{"name"};

  my $label = Gtk2::Button->new();
  my   $sublabel = Gtk2::Label->new;
  if($aliases{$labelText})
  {
    print("processing an alias... ".$labelText."\n");
    #$label->set_markup('<span foreground="blue">'.$labelText.'</span>');
    $sublabel->set_markup('<span foreground="blue">'.$labelText.'</span>');

  }# if($aliases->{$labelText})
  else
  {
    $sublabel->set_text($labelText);
  }# else
  $sublabel->set_alignment (0.0, 0.5); # left-aligned
  $label->add ($sublabel);
  #$labelText = " ".$labelText;
  #my $label = Gtk2::Label->new(" ".$line->{"name"});
  #my $label = Gtk2::Button->new($labelText);
  #$label->set_justify('left');
  #$label->set_alignment(0,0);

  #$label->set_line_wrap(0 );
  $label->signal_connect( 'clicked' =>
      sub {
      ui_aliasdialog($kernel, $line->{"name"});
      }
      );
  ##$box->table_attach( $label, 0, 1, 0, 1,0,0,0,0);
  $next = $index+1;
  $box->attach($label, $x, ($x + 1), $index, $next,['expand','fill'] ,['expand','fill'] , 0, 0);
  $label->show();
  $table_line->{"name"} = $label;
  $x++;

  my $button = Gtk2::Button->new('D');
  $next = $index+1;
  $box->attach($button, $x, ($x + 1), $index, $next,['expand','fill'] ,['expand','fill'] , 0, 0);
  $button->signal_connect( 'clicked' =>
      sub {
      delseries(@_, $line->{"name"}, $kernel);
      }
      );
  $table_line->{"delete"} = $button;
  $button->show();
  $x++;

  #$label = Gtk2::Entry->new();
  #$label->set_text($line->{"min"});
  my $adj = new Gtk2::Adjustment($line->{"min"}, 0, 1025, 1, 10, 10 );
  $label = new Gtk2::SpinButton($adj,1,0);
  $adj->signal_connect( 'value_changed' => 
      sub { min_change(@_, $line->{"name"}, "min",$kernel);});
  #$adj->signal_connect( 'value_changed', sub { $kernel->post("gtkSeries", "ev_change") }, $line->{"name"}, "min", $label->get_value_as_int());
  $box->attach($label, $x, ($x + 1), $index, $next,['expand','fill'] ,['expand','fill'] , 0, 0);
  $label->show();
  $table_line->{"min"} = $label;
  $x++;

  $adj = new Gtk2::Adjustment($line->{"max"}, 0, 1025, 1, 10, 10 );
  $label = new Gtk2::SpinButton($adj,1,0);
  $adj->signal_connect( 'value_changed'=> 
      sub{ min_change(@_, $line->{"name"}, "max",$kernel);} );
  $box->attach($label, $x, ($x + 1), $index, $next,['expand','fill'] ,['expand','fill'] , 0, 0);
  $label->show();
  $table_line->{"max"} = $label;
  $x++;

  my $text = " ";
  $text = $line->{"other"} if($line->{"other"});
  #$label = new Gtk2::Text( undef, undef );
  #$label->set_editable(1 );
  #$label->insert(undef, undef, undef, $text);
  $label = new Gtk2::Entry();
  $label->set_text($text);
  #$label->set_alignment(0,0);
  #$label->set_line_wrap(1 );
  #$label->set_editable(1 );
  $box->attach($label, $x, ($x + 1), $index, $next,['expand','fill'] ,['expand','fill'] , 0, 0);
  $label->signal_connect( 'activate' => 
      sub { add_episodes(@_, $line->{"name"}, $kernel);});
  $label->show();
  $table_line->{"other"} = $label;
  $x++;

  #print("added ".$line->{"name"}." at ".$index."x$next\n");
  return($table_line);
}#sub add_table_line
##############################################################################
=pod

=item default 

got soemthing from the default signal....

=cut

##############################################################################
sub default
{
  my ( $kernel, $arg, $heap ) = @_[ KERNEL, ARG0, HEAP ];
  #print("got somethign through default{$arg}:".Dumper($arg)."\n");
  my $msg = $arg;
  $msg = $$msg if(ref($msg) eq 'SCALAR');

  if(ref($msg) eq 'ARRAY')
  {
    if(scalar(@$msg) >0)
    {
      $msg = Dumper($msg);
    }
    else
    {
      $msg = "";
    }# else
  }# if(ref($msg) eq 'ARRAY' && scalar(@$msg) >0)

  if($msg)
  {
    if($msg =~ /^-----  Dumping settings/)
    {
       create_setdialog($msg,$kernel);
    }# if($msg =~ /^-----  Dumping settings/)
    elsif($msg =~ /^print\[(.*?)\]: (.*)/)
    {
      my $process = $1;
      my $line = $2;
      #print("got print request for process $1 with text: $2\n");
      if($line =~ /^BT: (.*)/)
      {
        $line = $1;
        $line =~ /(.*?)\s+%:(.*?)\s+ETA: (.*?) D:(.*?) U:(.*?)/;
        $transfers->{$1}->{"completion"} = $2/100.0;
        $transfers->{$1}->{"name"} = $1;
        $transfers->{$1}->{"eta"} = $3;
        $transfers->{$1}->{"downrate"} = $4;
        $transfers->{$1}->{"uprate"} = $5;
        #print("updating transfers for : $1\n");
        $kernel->yield('updatetransfers',$transfers->{$1});
      }# if($line =~ /^BT: (.*)/)
      else
      {
        $line .= "\n" if(!($line =~ /\n$/));
        my $toinsert = [ "[$process] blue", $line  ];
        $printbuffer->add_text($toinsert);
        print("should have logged $line\n");
      }#else
    $kernel->yield("refresh");

  }# elsif($msg =~ /^print\[(.*?)\]: (.*)/)
  else
  {
    my $dialog = Gtk2::MessageDialog->new($heap->{main_window},
        [qw/destroy-with-parent/],
        'info',
        'ok',
        "Default: $msg");

    $dialog->run;
    $dialog->destroy;
  }
}#if($msg)
}# sub default
##############################################################################
=pod

=item gotMessage 

got something from the default signal....

=cut

##############################################################################
sub gotMessage
{
  my ( $kernel, $hash, $heap ) = @_[ KERNEL, ARG0, HEAP ];
  #print("gotMessage got $$hash\n");
  if($$hash =~ /^\s*---- Series/i)
  {
    #got the series listing!
    my @rawseries = split(/\n/,$$hash);
    #remove enclosing rubbish
    shift(@rawseries);
    pop(@rawseries);
    foreach my $line (@rawseries)
    {
      if( $line =~ /(.*?)'(.*?)'(\d+)-(\d+)\s+other:\s+(.*)/)
      {
	my $entry = {
	  "name" => $2, "min" => $3, "max" => $4,
	  "other" => $5
	};
	my $head = $1;
	#changed added alias parsing
	$entry->{"alias"} = 1 if($head =~ /alias/i);
	$series{$1} = $entry;
      }# if( $line =~ /.*?'(.*?)'(\d+)-(\d+)\s+other:\s+(.*)/)
      elsif( $line =~ /.*?'(.*?)'(\d+)-(\d+)/)
      {
	my $entry = {
	  "name" => $1, "min" => $2, "max" => $3
	};
	#push(@series,$entry);
	$series{$1} = $entry;
      }#else
      else
      {
	print("unparsable: $line\n");
      }# else

    }# foreach my $line (@rawseries)
    #print("series: ".Dumper(\@series)."\n");
    $_[KERNEL]->post("gtkSeries", "fillup"); 
  }# if($$hash =~ /^\s*---- Series/i)
  elsif($$hash =~ /^\s*---- Aliases/i)
  {
    #got the series listing!
    my @rawseries = split(/\n/,$$hash);
    #remove enclosing rubbish
    shift(@rawseries);
    pop(@rawseries);
    foreach my $line (@rawseries)
    {
      if( $line =~ /^(.*?)'(.*?)'\s+for\s+(.*)$/)
      {
	$aliases{$2} = $3;
    }# if( $line =~ /.*?'(.*?)'(\d+)-(\d+)\s+other:\s+(.*)/)
    else
    {
      print("unparsable: $line\n");
    }# else
  }# foreach my $line (@rawseries)
    #print( "extracted aliases: ".Dumper(\%aliases)."\n" );
}# if($$hash =~ /^\s*---- Series/i)
else
{
  print("got unknown message $$hash\n");
  my $msg = $hash;
  $msg = $$msg if(ref($msg) eq 'SCALAR');
  my $dialog = Gtk2::MessageDialog->new($heap->{main_window},
      [qw/destroy-with-parent/],
      'info',
      'ok',
      "Message: $msg");

  $dialog->run;
  $dialog->destroy;
}# else
}# sub gotMessage
##############################################################################
=pod

=item newSeries 

callback for the GTK stuff, this method is called when an event occurs on one of the spin buttons

=cut

##############################################################################
sub newSeries 
{
my($kernel,$heap) = @_;

if($heap->{"newinput"}->{"pattern"}->get_text())
{
  my $msg = "series ";
  $msg .= $heap->{"newinput"}->{"pattern"}->get_text();
  $msg .= " ".$heap->{"newinput"}->{"max"}->get_value();
  $msg .= " ".$heap->{"newinput"}->{"min"}->get_value();
  $msg .= " ".$heap->{"newinput"}->{"other"}->get_text() if($heap->{"newinput"}->{"other"}->get_text());
  print("would send $msg\n");

  my $job = {
    "requests"	=> [$msg],
    "session"		=> "gtkSeries",
    "event"		=> "default",
    "msgevent"	=> "default",
  };
  if($kernel)
  {
    $kernel->post($querySession, "doneParsing", $job);
  }
  else
  {
    print("no kernel where to put '$msg'\n");
  }# else
}# if($heap->{"newinput"}->{"pattern"}->get_text())
}# sub newSeries 
##############################################################################
=pod

=item updateServerSettings 

Update the Settings the server proposed

=cut

##############################################################################
sub updateServerSettings 
{
my($kernel,$form,$setDialog,$switches) = @_;

my @newSettings = $form->get_values();
my @requests;
foreach my $key (sort(keys(%$switches)))
{
  my $newval = shift(@newSettings);
  if($newval ne $switches->{$key})
  {
    push(@requests,"set $key $newval");
  }# if($newval ne $switches{$key})
}# foreach my $set (@lines)

$setDialog->destroy(); 
my $job = {
  "requests"	=> \@requests,
  "session"		=> "gtkSeries",
  "event"		=> "default",
  "msgevent"	=> "default",
};
if($kernel)
{
  $kernel->post($querySession, "doneParsing", $job);
}
else
{
  print("no kernel where to put 'updateServerSettings'\n");
}# else

}#sub updateServerSettings 
##############################################################################
=pod

=item refresh

if visible update the debug/download window

=cut
######################################################################
sub refresh
{
my ( $kernel ) = @_[ KERNEL, ARG0 ];

}# sub refresh
##############################################################################
=pod

=item ui_statusdialog 

show a status dialog with debugging messages and status of actual transfers

=cut

##############################################################################
sub ui_statusdialog 
{
  my($kernel,$heap) = @_;
  if($windows->{"status"})
  {
    $windows->{"status"}->destroy();
    delete $windows->{"status"};
  }#if($windows->{"status"})
  else
  {
    my $statusDialog = new Gtk2::Dialog;

    $statusDialog->set_title('Status');
    $statusDialog->set_border_width(8);
    $statusDialog->set_size_request(400,400);
    $statusDialog->set_modal(FALSE);
    $statusDialog->set_resizable(TRUE);
    $statusDialog->add_buttons('close' => 0);
    $statusDialog->vbox->set_spacing(8);
    #$statusDialog->vbox->pack_start($label, TRUE, TRUE, 0);
    my $scroll_window = Gtk2::ScrolledWindow->new();
    $scroll_window->set_policy ('automatic', 'automatic');
    #$statusDialog->vbox->pack_start($form->get_widget, TRUE, TRUE, 0);
    $statusDialog->vbox->pack_start($scroll_window, TRUE, TRUE, 0);
    my $view1;
    $view1 = Gtk2::TextView->new_with_buffer($printbuffer);
    $scroll_window->add_with_viewport($view1);
    $printbuffer->set_view($view1);

    my $toinsert = [ "SET VIEW on new widget red","\n" ];
    $printbuffer->add_text($toinsert);

    $statusDialog->signal_connect(response => 
	sub 
	{ 
	foreach my $atrans (keys(%$transfers))
	{
        $windows->{"progresssize"}->remove_widget($transfers->{$atrans}->{"bar"});
	delete $transfers->{$atrans}->{"bar"};
	}# foreach my $atrans (keys(%$transfers))
	$printbuffer->set_view();

	delete $windows->{"status"}; 
	$statusDialog->destroy(); 
	});
    #updateServerSettings($kernel,$form,$statusDialog,\%switches);
    $windows->{"status"} = $statusDialog;

    foreach my $atrans (keys(%$transfers))
    {
      my $transData = $transfers->{$atrans};
      my $bar = $transData->{"bar"};
      if(!$bar)
      {
	#print("adding new progressbar\n");
	$bar = create_progressbar($atrans,$transData,$statusDialog);
        $windows->{"progresssize"}->add_widget($bar);
      }# if($atrans->{"bar"})
      $bar->set_fraction($transData->{"completion"});
      $transData->{"etalabel"}->set_text("ETA ".$transData->{"eta"});
      $transData->{"dratelabel"}->set_text($transData->{"downrate"});
      #print("create $atrans %=".$transData->{"completion"}."\n");

    }# foreach my $atrans (keys(%$transfers))

    $statusDialog->show_all;
  }#else
}# sub ui_statusdialog 
##############################################################################
=pod

=item updatetransfers

event to update the progress bars

=cut

##############################################################################
sub updatetransfers 
{
  my ($kernel, $ref) = @_[KERNEL,ARG0];
  my $bar = $ref->{"bar"};
  if($windows->{"status"} )
  {
    if(!$bar)
    {
      $bar = create_progressbar($ref->{"name"},$ref,$windows->{"status"});
      $windows->{"progresssize"}->add_widget($bar);
    }
  }

  if($bar)
  {
    #print("setting fraction to ".$transfers->{$1}->{"completion"}."\n");
    $bar->set_fraction($ref->{"completion"});
    $ref->{"etalabel"}->set_text("ETA ".$ref->{"eta"});
    $ref->{"dratelabel"}->set_text($ref->{"downrate"});
  }# if(!$bar)
}#sub updatetransfers 
##############################################################################
=pod

=item create_progressbar 

create a new progress bar for a new transfer

=cut

##############################################################################
sub create_progressbar 
{
  my($name,$transData,$statusDialog) = @_;
  #print("adding new progressbar\n");
  my $bar = Gtk2::ProgressBar->new();
  $transData->{"bar"} = $bar;
  $transData->{"etalabel"} = new Gtk2::Label;
  $transData->{"dratelabel"} = new Gtk2::Label;
  $bar->set_text($name);

  my $progressbox = new Gtk2::HBox(0,0);
  $progressbox->add($bar);
  $progressbox->add($transData->{"dratelabel"});
  $progressbox->add($transData->{"etalabel"});

  $statusDialog->vbox->pack_end($progressbox,0,0,0);
  $progressbox->show_all();
  $progressbox->queue_resize();
  return $bar;
}#sub create_progressbar 
##############################################################################
=pod

=item create_setdialog 

fire up the dialog that displays and manages settings

=cut

##############################################################################
sub create_setdialog 
{
  my ($msg,$kernel) = @_;

  my @lines = split(/\n/,$msg);
  shift @lines;
  pop @lines;

  my %switches;

  foreach my $set (@lines)
  {
    my ($key,$value) = split(/ = /,$set,2);
    $value =~ s/^'(.*)'$/$1/;
    $switches{$key} = $value;
  }# foreach my $set (@lines)

  my @items;
  foreach my $key (sort(keys(%switches)))
  {
    push(@items, 
	{
	text		=> $key,
	type		=> 'string',
	value		=> $switches{$key},
	}
	);
  }# foreach my $set (@lines)

  use Gtk2::FormFactory;
  my $form = Gtk2::FormFactory->new;

  $form->set_items(@items);
  $form->set_label_align('left', 'left');
  $form->generate;

  my $setDialog = new Gtk2::Dialog;

  $setDialog->set_title('Settings');
  $setDialog->set_border_width(8);
  $setDialog->set_size_request(400,400);
  $setDialog->set_modal(TRUE);
  #$setDialog->set_resizable(FALSE);
  $setDialog->add_buttons('gtk-ok' => 0);
  $setDialog->vbox->set_spacing(8);
  #$setDialog->vbox->pack_start($label, TRUE, TRUE, 0);
  my $scroll_window = Gtk2::ScrolledWindow->new();
  #$setDialog->vbox->pack_start($form->get_widget, TRUE, TRUE, 0);
  $setDialog->vbox->pack_start($scroll_window, TRUE, TRUE, 0);
  $scroll_window->add_with_viewport($form->get_widget);
  $setDialog->show_all;
  $setDialog->signal_connect(response => sub { 
      updateServerSettings($kernel,$form,$setDialog,\%switches);
      });

}#sub create_setdialog 
##############################################################################
=pod

=item ui_aliasdialog 

show a alias dialog with debugging messages and alias of actual transfers

=cut

##############################################################################
sub ui_aliasdialog 
{
  my($kernel, $orgpattern) = @_;

  my $aliasDialog = new Gtk2::Dialog;

  $aliasDialog->set_title('Making an alias for '.$orgpattern);
  $aliasDialog->set_border_width(8);
  $aliasDialog->set_size_request(400,72);
  $aliasDialog->set_modal(FALSE);
  $aliasDialog->set_resizable(TRUE);
  $aliasDialog->add_buttons('ok' => 1);
  $aliasDialog->add_buttons('cancel' => 0);
  $aliasDialog->vbox->set_spacing(8);
  #$aliasDialog->vbox->pack_start($label, TRUE, TRUE, 0);

  my $patternBox = new Gtk2::Entry();
  $aliasDialog->vbox->pack_start($patternBox, TRUE, TRUE, 0);


  $aliasDialog->signal_connect(response => 
      sub 
      { 
      my ($theDialog, $buttonState) = @_;
      #delete $windows->{"alias"}; 
      if($buttonState )
      {
	my @children = $theDialog->vbox->get_children();
	my $entry = shift(@children);
	my $title = $theDialog->get_title();
	my $data = $entry->get_text();
	$title =~ s/Making an alias for //g;
	my @request = ( 
	    "alias $data $title",
	    );
	my $job = {
	  "requests"	=> \@request,
	  "session"		=> "gtkSeries",
	  "event"		=> "default",
	  "msgevent"	=> "default",
	};
	if($kernel)
	{
	  $kernel->post($querySession, "doneParsing", $job);
	}
	else
	{
	  print("no kernel where to put 'alias $data $title'\n");
	}# else
      }# if($buttonState )
      else
      {
	print("cancel, ignoring the return stuff\n");
      }# else
      $theDialog->destroy(); 
      });
  #updateServerSettings($kernel,$form,$aliasDialog,\%switches);
  #$windows->{"alias"} = $aliasDialog;
  $aliasDialog->show_all;
}# sub ui_aliasdialog 
__END__


=back

=head1 THANKS

I. Wronsky came up with the concept for leverloda
This gave B. Boettcher the incentive to begin this work, the
famous itching paradigm!

=head1 SEE ALSO

http://www.geocities.com/iwronsky/
http://poe.perl.org/

=cut

